@@ -1,30 +1,38 @@
 # Texinfo.pm: format Pod as Texinfo.
 #
-# Copyright 2011, 2012, 2013, 2014, 2022 Free Software Foundation, Inc.
-# 
+# Copyright 2011-2024 Free Software Foundation, Inc.
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License,
 # or (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-# 
+#
 # Original author: Patrice Dumas <pertusus@free.fr>
 # Parts from L<Pod::Simple::HTML>.
 #
 #
-# The code could easily be used directly as a Pod::Simple subclass
-# by renaming _texinfo_handle_element_start and the two other
-# similar functions as _handle_element_start, or as a
-# Pod::Simple::SimpleTree subclass, using _convert_pod_simple_tree.
-# We prefer a Pod::Simple::PullParser subclassing to be able to use
-# get_short_title().
+# The code is organized such that it is easy to use any Pod::Simple
+# interface:
+# * We mainly use Pod::Simple::PullParser subclassing to be able to use
+#   get_short_title();  _convert_pod_tokens is the main function used to get
+#   PullParser tokens.
+# * The code could also be used directly as a Pod::Simple subclass by renaming
+#   _texinfo_handle_element_start and the two other similar functions as
+#   _handle_element_start ad similar.
+# * The code can also be used as a Pod::Simple::SimpleTree subclass, by
+#   using _convert_pod_simple_tree.  The possibility to convert SimpleTree
+#   is actually used in the code for L<> formatting.
+#
+# bare_output flag described in Pod::Simple::Subclassing is taken into
+# account.
 #
 #
 # TODO: it could be relevant to convert L<...> referring to external modules to
@@ -34,7 +42,7 @@
 
 package Pod::Simple::Texinfo;
 
-require 5;
+use 5.006;
 use strict;
 #no autovivification qw(fetch delete exists store strict);
 
@@ -45,23 +53,16 @@ use Pod::Simple::PullParser ();
 # for parselink()
 #use Pod::ParseLink;
 
-use Texinfo::Convert::NodeNameNormalization qw(normalize_node);
-use Texinfo::Parser qw(parse_texi_line parse_texi_text);
+use Texinfo::Convert::NodeNameNormalization qw(convert_to_identifier);
+use Texinfo::Parser;
 use Texinfo::Convert::Texinfo;
 use Texinfo::Convert::TextContent;
-use Texinfo::Common qw(protect_colon_in_tree protect_comma_in_tree
-                                         protect_first_parenthesis);
-use Texinfo::Transformations qw(protect_hashchar_at_line_beginning
-                                          reference_to_arg_in_tree);
+use Texinfo::Document;
+use Texinfo::ManipulateTree;
+use Texinfo::Transformations;
 
-use vars qw(
-  @ISA $VERSION
-);
-
-@ISA = ('Pod::Simple::PullParser');
-$VERSION = '0.01';
-
-#use UNIVERSAL ();
+our @ISA = qw(Pod::Simple::PullParser);
+our $VERSION = '0.01';
 
 # Allows being called from the command line as
 # perl -w -MPod::Simple::Texinfo -e Pod::Simple::Texinfo::go thingy.pod
@@ -102,6 +103,8 @@ my @raw_formats = ('html', 'HTML', 'docbook', 'DocBook', 'texinfo',
 # from other Pod::Simple modules.  Creates accessor subroutine.
 __PACKAGE__->_accessorize(
   'texinfo_add_upper_sectioning_command',
+  'texinfo_debug',
+  'texinfo_generate_setfilename', # for standalone manuals
   'texinfo_internal_pod_manuals',
   'texinfo_man_url_prefix',
   'texinfo_main_command_sectioning_style',
@@ -122,15 +125,19 @@ sub new
   my $new = $class->SUPER::new(@_);
   $new->accept_targets(@raw_formats);
   $new->preserve_whitespace(1);
+  $new->texinfo_debug(0);
+  # TODO set to 0 when the time has come.
+  #$new->texinfo_generate_setfilename(0);
   $new->texinfo_section_nodes(0);
   $new->texinfo_sectioning_base_level($sectioning_base_level);
   $new->texinfo_man_url_prefix($man_url_prefix);
   $new->texinfo_sectioning_style($sectioning_style);
   $new->texinfo_add_upper_sectioning_command(1);
-  #$new->{'texinfo_nodes'} = {};
   return $new;
 }
 
+# This needs to be defined so that users can call parse_file and other
+# similar Pod::Simple/Pod::Simple::PullParser methods.
 sub run
 {
   my $self = shift;
@@ -226,39 +233,40 @@ sub _preamble($)
   if ($self->texinfo_sectioning_base_level() == 0) {
     #print STDERR "$fh\n";
     print $fh '\input texinfo'."\n";
-    my $setfilename;
-    if (defined($self->texinfo_short_title())) {
-      $setfilename = _pod_title_to_file_name($self->texinfo_short_title());
-    } else {
-      # FIXME maybe output filename would be better than source_filename?
-      my $source_filename = $self->source_filename();
-      if (defined($source_filename) and $source_filename ne '') {
-        if ($source_filename eq '-') {
-          $setfilename = $STDIN_DOCU_NAME;
-        } else {
-          $setfilename = $source_filename;
-          $setfilename =~ s/\.(pod|pm)$//i;
+    if ($self->texinfo_generate_setfilename()) {
+      my $setfilename;
+      if (defined($self->texinfo_short_title())) {
+        $setfilename = pod_title_to_file_name($self->texinfo_short_title());
+      } else {
+        my $source_filename = $self->source_filename();
+        if (defined($source_filename) and $source_filename ne '') {
+          if ($source_filename eq '-') {
+            $setfilename = $STDIN_DOCU_NAME;
+          } else {
+            $setfilename = $source_filename;
+            $setfilename =~ s/\.(pod|pm)$//i;
+          }
         }
       }
-    }
-    if (defined($setfilename) and $setfilename =~ m/\S/) {
-      $setfilename = _protect_text($setfilename, 1, 1);
-      $setfilename .= '.info';
-      print $fh "\@setfilename $setfilename\n\n"
+      if (defined($setfilename) and $setfilename =~ m/\S/) {
+        $setfilename = protect_text($setfilename, 1, 1);
+        $setfilename .= '.info';
+        print $fh "\@setfilename $setfilename\n\n"
+      }
     }
 
     my $title = $self->get_title();
     if (defined($title) and $title =~ m/\S/) {
-      print $fh "\@settitle "._protect_text($title, 1)."\n\n";
+      print $fh "\@settitle ".protect_text($title, 1)."\n\n";
     }
     print $fh "\@node Top\n";
     if (defined($self->texinfo_short_title())) {
-      print $fh "\@top "._protect_text($self->texinfo_short_title(), 1)."\n\n";
+      print $fh "\@top ".protect_text($self->texinfo_short_title(), 1)."\n\n";
     }
   } elsif (defined($self->texinfo_short_title())
            and $self->texinfo_add_upper_sectioning_command()) {
     my $level = $self->texinfo_sectioning_base_level() - 1;
-    my $name = _protect_text($self->texinfo_short_title(), 1, 1);
+    my $name = protect_text($self->texinfo_short_title(), 1, 1);
     my $node_name = _prepare_anchor($self, $name);
 
     my $anchor = '';
@@ -272,7 +280,7 @@ sub _preamble($)
       }
     }
     print $fh "$node\@$self->{'texinfo_sectioning_main_command'}->[$level] "
-       ._protect_text($self->texinfo_short_title(), 1)."\n$anchor\n";
+       .protect_text($self->texinfo_short_title(), 1)."\n$anchor\n";
   }
 }
 
@@ -310,7 +318,8 @@ sub _end_context($)
   return ($previous_context->{'text'}, $previous_context->{'out'});
 }
 
-sub _protect_text($;$$)
+# Also used in pod2texi.pl but not public.
+sub protect_text($;$$)
 {
   my $text = shift;
   my $remove_new_lines = shift;
@@ -337,9 +346,11 @@ sub _protect_text($;$$)
   return $text;
 }
 
-sub _pod_title_to_file_name($)
+# Used in pod2texi.pl but not public
+sub pod_title_to_file_name($)
 {
   my $name = shift;
+  $name =~ s/[\n\r]//g;
   $name =~ s/\s+/_/g;
   $name =~ s/::/-/g;
   $name =~ s/[^\w\.-]//g;
@@ -350,16 +361,20 @@ sub _pod_title_to_file_name($)
 sub _protect_comma($)
 {
   my $texinfo = shift;
-  my $tree = parse_texi_line(undef, $texinfo);
-  $tree = protect_comma_in_tree($tree);
+  my $parser = Texinfo::Parser::parser();
+  my $tree = $parser->parse_texi_line($texinfo);
+  Texinfo::ManipulateTree::protect_comma_in_tree($tree);
+  $tree = Texinfo::Document::rebuild_tree($tree);
   return Texinfo::Convert::Texinfo::convert_to_texinfo($tree);
 }
 
 sub _protect_colon($)
 {
   my $texinfo = shift;
-  my $tree = parse_texi_line(undef, $texinfo);
-  $tree = protect_colon_in_tree($tree);
+  my $parser = Texinfo::Parser::parser();
+  my $tree = $parser->parse_texi_line($texinfo);
+  Texinfo::ManipulateTree::protect_colon_in_tree($tree);
+  $tree = Texinfo::Document::rebuild_tree($tree);
   return Texinfo::Convert::Texinfo::convert_to_texinfo($tree);
 }
 
@@ -368,8 +383,12 @@ sub _protect_hashchar($)
   my $texinfo = shift;
   # protect # first in line
   if ($texinfo =~ /#/) {
-    my $tree = parse_texi_text(undef, $texinfo);
-    protect_hashchar_at_line_beginning(undef, undef, $tree);
+    my $parser = Texinfo::Parser::parser();
+    my $document = $parser->parse_texi_piece($texinfo);
+    my $tree = $document->tree();
+    Texinfo::Transformations::protect_hashchar_at_line_beginning($tree);
+    # rebuild the tree
+    $tree = $document->tree();
     return Texinfo::Convert::Texinfo::convert_to_texinfo($tree);
   } else {
     return $texinfo;
@@ -379,8 +398,12 @@ sub _protect_hashchar($)
 sub _reference_to_text_in_texi($)
 {
   my $texinfo = shift;
-  my $tree = parse_texi_text(undef, $texinfo);
-  reference_to_arg_in_tree($tree);
+  my $parser = Texinfo::Parser::parser();
+  my $document = $parser->parse_texi_piece($texinfo);
+  my $tree = $document->tree();
+  Texinfo::Transformations::reference_to_arg_in_tree($tree);
+  # rebuild the tree
+  $tree = $document->tree();
   return Texinfo::Convert::Texinfo::convert_to_texinfo($tree);
 }
 
@@ -393,19 +416,52 @@ sub _prepend_internal_section_manual($$$;$$)
   my $in_code = shift;
 
   if (defined($manual) and $base_level > 0) {
-    $manual = _protect_text($manual, 1, $in_code) if ($protect_text);
+    $manual = protect_text($manual, 1, $in_code) if ($protect_text);
     return "$manual $section";
   } else {
     return $section;
   }
 }
 
-sub _normalize_texinfo_name($$)
+# also used in pod2texi.pl, not public.
+sub print_texinfo_errors($$;$)
+{
+  my $error_messages = shift;
+  my $error_count = shift;
+  my $location = shift;
+
+  foreach my $error_message (@$error_messages) {
+    my $type_string;
+    if ($error_message->{'type'} eq 'error') {
+      $type_string = 'ERROR';
+    } else {
+      $type_string = 'WARNING';
+    }
+    my $location_string;
+    if (defined($location)) {
+      $location_string = "$location:";
+    } else {
+      $location_string = '';
+    }
+    if (defined($error_message->{'file_name'})) {
+      $location_string .= "$error_message->{'file_name'}:";
+    }
+    if (defined($error_message->{'line_nr'})) {
+      $location_string .= "$error_message->{'line_nr'}:";
+    }
+    $location_string .= ' ' unless ($location_string eq '');
+    warn "$type_string: ${location_string}$error_message->{'error_line'}";
+  }
+}
+
+sub _normalize_texinfo_name($$;$)
 {
   # Pod may be more forgiven than Texinfo, so we go through
   # a normalization, by parsing and converting back to Texinfo
   my $name = shift;
   my $command = shift;
+  my $debug = shift;
+
   my $texinfo_text;
   if ($command eq 'anchor') {
     $texinfo_text = "\@anchor{$name}";
@@ -417,44 +473,39 @@ sub _normalize_texinfo_name($$)
     }
     $texinfo_text = "\@$command $name\n";
   }
-  my $parser = Texinfo::Parser::parser();
-  my $tree = $parser->parse_texi_piece($texinfo_text);
+
+  my $parser_options = {};
+  if (defined($debug) and $debug > 4) {
+    $parser_options->{'DEBUG'} = $debug - 4;
+  }
+  my $parser = Texinfo::Parser::parser($parser_options);
+  my $document = $parser->parse_texi_piece($texinfo_text);
+
+  my $tree = $document->tree();
+
+  my ($error_messages, $error_count) = $document->parser_errors();
+
   if (!defined($tree)) {
     my $texinfo_text_str = $texinfo_text;
     chomp($texinfo_text_str);
     warn "ERROR: Texinfo parsing failed for: $texinfo_text_str\n";
-    my $registrar = $parser->registered_errors();
-    my ($parser_errors, $parser_error_count) = $registrar->errors();
-    foreach my $error_message (@$parser_errors) {
-      if ($error_message->{'type'} eq 'error') {
-        warn "ERROR: $error_message->{'error_line'}";
-      } else {
-        warn "WARNING: $error_message->{'error_line'}";
-      }
-    }
-    # FIXME Or undef, and callers check the return to be defined?
-    return '';
+    print_texinfo_errors($error_messages, $error_count);
+    return undef;
+  # use a high debug number, as the errors and warnings are likely to be
+  # redundant with the warnings and errors emitted when fixing the document
+  # and also because we go through Texinfo parsing and outputing as Texinfo
+  # not only to apply transformations, but also possibly to fix invalid
+  # constructs.
+  } elsif (defined($debug) and $debug > 3) {
+    print_texinfo_errors($error_messages, $error_count,
+                         '_normalize_texinfo_name');
   }
   if ($command eq 'anchor') {
-    # FIXME this works to find the anchor command only if
-    # the tree structure always leads to the interesting
-    # elements being first in the contents.
-    my $current = $tree;
-    while ((not exists($current->{'cmdname'})
-            or $current->{'cmdname'} ne 'anchor')
-           and $current->{'contents'}
-           and scalar(@{$current->{'contents'}})) {
-      $current = $current->{'contents'}->[0];
-    }
-    if (not exists($current->{'cmdname'}) or $current->{'cmdname'} ne 'anchor') {
-      cluck "BUG: could not find anchor: $texinfo_text";
-    } elsif ($current->{'args'}->[0]->{'contents'}) {
-      my $protected_contents
-          = protect_first_parenthesis($current->{'args'}->[0]->{'contents'});
-      $current->{'args'}->[0]->{'contents'} = $protected_contents;
-    }
+    Texinfo::Transformations::protect_first_parenthesis_in_targets($tree);
+    # rebuild the tree
+    $tree = $document->tree();
   }
-  my $fixed_text = Texinfo::Convert::Texinfo::convert_to_texinfo($tree, 1);
+  my $fixed_text = Texinfo::Convert::Texinfo::convert_to_texinfo($tree);
   my $result = $fixed_text;
   if ($command eq 'anchor') {
     $result =~ s/^\@anchor\{(.*)\}$/$1/s;
@@ -484,14 +535,16 @@ sub _prepare_anchor($$)
   my $self = shift;
   my $texinfo_node_name = shift;
 
-  my $node = _normalize_texinfo_name($texinfo_node_name, 'anchor');
+  my $node = _normalize_texinfo_name($texinfo_node_name, 'anchor',
+                                     $self->texinfo_debug());
 
-  if ($node !~ /\S/) {
+  if (!defined($node) or $node !~ /\S/) {
     return '';
   }
   # Now we know that we have something.
-  my $node_tree = parse_texi_line(undef, $node);
-  my $normalized_base = normalize_node($node_tree);
+  my $parser = Texinfo::Parser::parser();
+  my $node_tree = $parser->parse_texi_line($node);
+  my $normalized_base = convert_to_identifier($node_tree);
   my $normalized = $normalized_base;
   my $number_appended = 0;
   while ($self->{'texinfo_nodes'}->{$normalized}) {
@@ -501,12 +554,13 @@ sub _prepare_anchor($$)
   my $node_name;
   if ($number_appended) {
     $texinfo_node_name = "$node $number_appended";
-    $node_tree = parse_texi_line(undef, $texinfo_node_name);
+    $node_tree = $parser->parse_texi_line($texinfo_node_name);
   }
-  $node_tree = protect_comma_in_tree($node_tree);
-  $node_tree = protect_colon_in_tree($node_tree);
+  Texinfo::ManipulateTree::protect_comma_in_tree($node_tree);
+  Texinfo::ManipulateTree::protect_colon_in_tree($node_tree);
+  $node_tree = Texinfo::Document::rebuild_tree($node_tree);
   $self->{'texinfo_nodes'}->{$normalized} = $node_tree;
-  my $final_node_name = Texinfo::Convert::Texinfo::convert_to_texinfo($node_tree, 1);
+  my $final_node_name = Texinfo::Convert::Texinfo::convert_to_texinfo($node_tree);
   return $final_node_name;
 }
 
@@ -575,6 +629,8 @@ sub _texinfo_handle_element_start($$$)
   my $tagname = shift;
   my $attr_hash = shift;
 
+  my $debug = $self->texinfo_debug();
+
   my $fh = $self->{'output_fh'};
 
   # unset ignoring spaces right after <X> if there is a following tag,
@@ -615,18 +671,19 @@ sub _texinfo_handle_element_start($$$)
           $section = 1 if (!defined($section));
           # it is unlikely that there is a comma because of _url_escape
           # but to be sure there is still a call to _protect_comma.
-          $url_arg = _protect_comma(_protect_text(
-                                       $self->texinfo_man_url_prefix()
-                                          ."$section/"._url_escape($page), 0, 1));
+          $url_arg = _protect_comma(protect_text(
+                                     $self->texinfo_man_url_prefix()
+                                        ."$section/"._url_escape($page), 0, 1));
         } else {
           $url_arg = '';
         }
-        $replacement_arg = _protect_text($replacement_arg);
-        _output($fh, $self->{'texinfo_accumulated'}, "\@url{$url_arg,, $replacement_arg}");
+        $replacement_arg = protect_text($replacement_arg);
+        _output($fh, $self->{'texinfo_accumulated'},
+                                         "\@url{$url_arg,, $replacement_arg}");
       } elsif ($linktype eq 'url') {
         # NOTE: the .'' is here to force the $token->attr to be a real
         # string and not an object.
-        $url_arg = _protect_comma(_protect_text($attr_hash->{'to'}.'', 0, 1));
+        $url_arg = _protect_comma(protect_text($attr_hash->{'to'}.'', 0, 1));
       } elsif ($linktype eq 'pod') {
         # The section is available from $attr_hash->{'section'} as a
         # tree (not a token, a Pod::Simple::SimpleTree), or as a plain text
@@ -636,9 +693,9 @@ sub _texinfo_handle_element_start($$$)
         # on the cases, to 'name', or '"section" in name' (based on perlpodspec)
         # which is not practical for conversion to Texinfo as section and
         # name should be available separately, both converted to Texinfo.
-        # It is possible to get the equivalent parsing in term of pod strings with
-        # parselink(), which returns the same as the pullparser argument as
-        # text, but also returns separately the section and name.
+        # It is possible to get the equivalent parsing in term of pod strings
+        # with parselink(), which returns the same as the pullparser argument
+        # as text, but also returns separately the section and name.
         # However, it is not possible to simply convert the section or
         # name string with the Texinfo pullparser parser, as a full pod text is
         # expected, starting whith a =head* while we would want to parse a
@@ -656,6 +713,7 @@ sub _texinfo_handle_element_start($$$)
           $self->_convert_pod_simple_tree($section);
           ($section_texi, $section_out)
             = _end_context($self->{'texinfo_accumulated'});
+
           # coerce to string
           $section_text = $section.'';
         }
@@ -666,6 +724,7 @@ sub _texinfo_handle_element_start($$$)
           $self->_convert_pod_simple_tree($manual);
           ($manual_texi, $manual_out)
             = _end_context($self->{'texinfo_accumulated'});
+
           # coerce to string
           $manual_text = $manual.'';
         }
@@ -692,12 +751,13 @@ sub _texinfo_handle_element_start($$$)
             # use plain text string without formatting to match with what should
             # be given through texinfo_internal_pod_manuals().
             if ($self->{'texinfo_internal_pod_manuals_hash'}->{$manual_text}) {
+              # should always be the first section in pods
               $section_texi = 'NAME';
               # use the manual name as texinfo section name, otherwise
               # it will be the section associated with the node, which is
               # the non informative 'NAME' section name
               $texinfo_section = _normalize_texinfo_name(
-                 _protect_comma($manual_texi), 'section');
+                        _protect_comma($manual_texi), 'section', $debug);
             }
           }
           # use plain text string without formatting to match with what should
@@ -707,8 +767,8 @@ sub _texinfo_handle_element_start($$$)
              _prepend_internal_section_manual($manual_texi, $section_texi,
                                  $self->texinfo_sectioning_base_level());
           } else {
-            $texinfo_manual = _protect_text(
-                        _pod_title_to_file_name($manual_text), 0, 1);
+            $texinfo_manual = protect_text(
+                        pod_title_to_file_name($manual_text), 0, 1);
             if (defined($section)) {
               $texinfo_node = $section_texi;
             } else {
@@ -721,15 +781,17 @@ sub _texinfo_handle_element_start($$$)
                                  $self->texinfo_short_title(), $section_texi,
                                  $self->texinfo_sectioning_base_level(), 1, 1);
           $texinfo_section = _normalize_texinfo_name(
-                                _protect_comma($section_texi), 'section');
+                           _protect_comma($section_texi), 'section', $debug);
           #print STDERR "L: internal: $texinfo_node/$texinfo_section\n";
         }
-        #print STDERR "L: not normalized node: $texinfo_node\n";
+        #print STDERR "L: not normalized node: '$texinfo_node'\n";
         $texinfo_node = _normalize_texinfo_name(
                 _protect_colon(
-                # FIXME remove end of lines?
-                _protect_comma($texinfo_node)), 'anchor');
-        #print STDERR "L: normalized node: $texinfo_node\n";
+                # empty lines are not valid in L<> in POD section, this is the
+                # same constraint as in @anchor
+                 _protect_comma($texinfo_node)), 'anchor', $debug);
+        $texinfo_node = '' if (!defined($texinfo_node));
+        #print STDERR "L: normalized node: '$texinfo_node'\n";
 
         # for pod, 'to' is the pod manual name.  Then 'section' is the
         # section.
@@ -747,15 +809,17 @@ sub _texinfo_handle_element_start($$$)
     _begin_context($self->{'texinfo_accumulated'}, $tagname);
   } elsif ($tag_commands{$tagname}) {
     _output($fh, $self->{'texinfo_accumulated'}, "\@$tag_commands{$tagname}\{");
-    if ($Texinfo::Common::brace_code_commands{$tag_commands{$tagname}}) {
+    if ($Texinfo::Commands::brace_code_commands{$tag_commands{$tagname}}) {
       if (@{$self->{'texinfo_stack'}} and ref($self->{'texinfo_stack'}->[-1]) eq ''
-          and defined($self->{'texinfo_raw_format_commands'}->{$self->{'texinfo_stack'}->[-1]})) {
+          and defined($self->{'texinfo_raw_format_commands'}
+                                        ->{$self->{'texinfo_stack'}->[-1]})) {
         cluck "in $self->{'texinfo_stack'}->[-1]: $tagname $tag_commands{$tagname}";
       }
       push @{$self->{'texinfo_stack'}}, 'in_code';
     }
   } elsif ($environment_commands{$tagname}) {
-    _output($fh, $self->{'texinfo_accumulated'}, "\@$environment_commands{$tagname}\n");
+    _output($fh, $self->{'texinfo_accumulated'},
+                                         "\@$environment_commands{$tagname}\n");
     if ($tagname eq 'Verbatim') {
       push @{$self->{'texinfo_stack'}}, 'verbatim';
     }
@@ -786,26 +850,31 @@ sub _texinfo_handle_text($$)
   }
   my $result_text;
   if (@{$self->{'texinfo_stack'}} and ref($self->{'texinfo_stack'}->[-1]) eq ''
-      and ((defined($self->{'texinfo_raw_format_commands'}->{$self->{'texinfo_stack'}->[-1]})
-            and !$self->{'texinfo_raw_format_commands'}->{$self->{'texinfo_stack'}->[-1]})
+      and ((defined($self->{'texinfo_raw_format_commands'}
+                                           ->{$self->{'texinfo_stack'}->[-1]})
+            and !$self->{'texinfo_raw_format_commands'}
+                                           ->{$self->{'texinfo_stack'}->[-1]})
            or ($self->{'texinfo_stack'}->[-1] eq 'verbatim'))) {
     $result_text = $text;
   } else {
     if (@{$self->{'texinfo_stack'}} and ref($self->{'texinfo_stack'}->[-1]) eq ''
         and ($self->{'texinfo_raw_format_commands'}->{$self->{'texinfo_stack'}->[-1]})) {
-      $result_text = _protect_text($text, 0, 1);
-      $result_text =~ s/^(\s*)#(\s*(line)? (\d+)(( "([^"]+)")(\s+\d+)*)?\s*)$/$1\@hashchar{}$2/mg;
+      $result_text = protect_text($text, 0, 1);
+      $result_text
+  =~ s/^(\s*)#(\s*(line)? (\d+)(( "([^"]+)")(\s+\d+)*)?\s*)$/$1\@hashchar{}$2/mg;
     } else {
-      $result_text = _protect_text($text, 0,
-                (@{$self->{'texinfo_stack'}} and $self->{'texinfo_stack'}->[-1] eq 'in_code'));
+      $result_text = protect_text($text, 0,
+                           (@{$self->{'texinfo_stack'}}
+                            and $self->{'texinfo_stack'}->[-1] eq 'in_code'));
     }
   }
   #print STDERR "T: !$text!->!$result_text!\n";
   _output($fh, $self->{'texinfo_accumulated'}, $result_text);
 }
 
-# tp remain compatible with PullParser, the $attr_hash should not be
-# used, and everything passed through the stack.
+# to remain compatible with PullParser, the $attr_hash should not be
+# used, information should be gathered at the start of the tag,
+# and passed through the stack $self->{'texinfo_stack'}.
 sub _texinfo_handle_element_end($$$)
 {
   my $self = shift;
@@ -836,12 +905,15 @@ sub _texinfo_handle_element_end($$$)
         $result =~ s/^\s*//;
         $result =~ s/\s*$//;
 
-        $command_argument = _normalize_texinfo_name($result, $command);
-        if ($result =~ /\S/ and $command_argument !~ /\S/) {
+        $command_argument = _normalize_texinfo_name($result, $command,
+                                                    $self->texinfo_debug());
+        if ($result =~ /\S/
+            and (!defined($command_argument) or $command_argument !~ /\S/)) {
           # use some raw text if the expansion lead to empty Texinfo code
-          my $tree = parse_texi_line(undef, $result);
+          my $parser = Texinfo::Parser::parse();
+          my $tree = $parser->parse_texi_line($result);
           my $converter = Texinfo::Convert::TextContent->converter();
-          $command_argument = _protect_text($converter->convert_tree($tree));
+          $command_argument = protect_text($converter->convert_tree($tree));
         }
 
         if ($pod_head_commands_level{$tagname}
@@ -932,7 +1004,7 @@ sub _texinfo_handle_element_end($$$)
     }
   } elsif ($tag_commands{$tagname}) {
     _output($fh, $self->{'texinfo_accumulated'}, "}");
-    if ($Texinfo::Common::brace_code_commands{$tag_commands{$tagname}}) {
+    if ($Texinfo::Commands::brace_code_commands{$tag_commands{$tagname}}) {
       pop @{$self->{'texinfo_stack'}};
     }
   } elsif ($environment_commands{$tagname}) {
@@ -955,8 +1027,13 @@ sub _texinfo_handle_element_end($$$)
   }
 }
 
-# does not appear as parsed token
+# NOTE does not appear as parsed token
 # E entity/character
+
+# Dispatch PullParser tokens to the formatting functions.  The processing
+# is done in the functions called, and not directly in the function such
+# that it is easy to implement parsing through other Pod::Simple interfaces
+# too.
 sub _convert_pod_tokens($)
 {
   my $self = shift;
@@ -1048,38 +1125,66 @@ It supports producing a standalone manual per Pod (the default) or
 render the Pod as a chapter, see L</texinfo_sectioning_base_level>.
 
 C<@documentencoding> is not output, which is consistent with outputting
-Texinfo in utf8 in the caller.
+Texinfo in UTF-8 in the caller.
 
 =head1 METHODS
 
 =over
 
+=item new
+
+Initialize a parser object.
+
+=item run
+
+Run the parser.  In general, you should not use this method directly,
+but instead use C<parse_file> or similar methods from L<Pod::Simple>.
+
+=back
+
+You can set these attributes on the parser object before you call
+C<parse_file> (or a similar method) on it:
+
+=over
+
 =item texinfo_add_upper_sectioning_command
 
 If set (the default case), a sectioning command is added at the beginning
 of the output for the whole document, using the module name, at the level
-above the level set by L<texinfo_sectioning_base_level>.  So there will be
+above the level set by L</texinfo_sectioning_base_level>.  So there will be
 a C<@part> if the level is equal to 1, a C<@chapter> if the level is equal
 to 2 and so on and so forth.  If the base level is 0, a C<@top> command is
 output instead.
 
+=item texinfo_debug
+
+Debug level.  Mainly or only used to turn on Texinfo parsing debugging, when
+Texinfo obtained from POD is parsed as Texinfo code to be normalized or
+modified and to report associated Texinfo processing errors.  More information
+output with higher levels.  Default 0, no debugging information output.
+
+=item texinfo_generate_setfilename
+
+If set, generate a C<@setfilename> line in standalone manuals. Ignored
+unless L</texinfo_sectioning_base_level> is 0.
+
 =item texinfo_internal_pod_manuals
 
 The argument should be a reference on an array containing the short
 titles (usually the module names) of all the Pod documents that are
 converted together and should be internal in the Texinfo document obtained
 by including all those Pod documents.  References to those documents use
-the internal reference commands formatting in Texinfo.  The formatting commands
-should not be present in the short titles.
+the internal reference commands formatting in Texinfo.  Formatting commands
+should not be present in these short titles.
 
-Corresponds to L<texinfo_sectioning_base_level> set to anything else than 0.
+Relevant if L</texinfo_sectioning_base_level> is not set to 0.
 
 =item texinfo_main_command_sectioning_style
 
 Sectioning style for the main command appearing at the beginning of the output
-file if L<texinfo_sectioning_base_level> is anything else than 0.  Unset in the
-default case.  If unset, use L<texinfo_sectioning_style>, except for style
-C<heading>, for which the C<number> style is used in the default case.
+file if L</texinfo_sectioning_base_level> not 0.  Unset in the default case.
+If unset, use L</texinfo_sectioning_style>, except for style C<heading>, for
+which the C<numbered> style is used in the default case.
 
 =item texinfo_man_url_prefix
 
@@ -1089,13 +1194,14 @@ is C<http://man.he.net/man>.
 =item texinfo_section_nodes
 
 If set, add C<@node> and not C<@anchor> for each sectioning command.
+Set to 0 in the default case.
 
 =item texinfo_sectioning_base_level
 
 Sets the level of the head1 commands.  1 is for the @chapter/@unnumbered
 level.  If set to 0, the head1 commands level is still 1, but the output
 manual is considered to be a standalone manual.  If not 0, the Pod file is
-rendered as a fragment of a Texinfo manual.
+rendered as a fragment of a Texinfo manual.  Default is 0.
 
 =item texinfo_sectioning_style
 
@@ -1121,7 +1227,7 @@ L<Pod::Simple>. L<Pod::Simple::PullParser>. The Texinfo manual.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011, 2012 Free Software Foundation, Inc.
+Copyright (C) 2011-2024 Free Software Foundation, Inc.
 
 This library is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
