@@ -1,6 +1,6 @@
 # Texinfo.pm: output a Texinfo tree as Texinfo.
 #
-# Copyright 2010-2023 Free Software Foundation, Inc.
+# Copyright 2010-2024 Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,33 +20,55 @@
 
 package Texinfo::Convert::Texinfo;
 
-use 5.00405;
+use 5.006;
 use strict;
 
+# stop \s from matching non-ASCII spaces, etc.  \p{...} can still be
+# used to match Unicode character classes.
+use if $] >= 5.014, re => '/a';
+
 # To check if there is no erroneous autovivification
 #no autovivification qw(fetch delete exists store strict);
 
 use Carp qw(cluck confess);
 
+use Texinfo::Convert::ConvertXS;
+use Texinfo::XSLoader;
+
 # commands definitions
 use Texinfo::Commands;
 # get_label_element
 use Texinfo::Common;
 
 require Exporter;
-use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);
-@ISA = qw(Exporter);
+our @ISA = qw(Exporter);
 
-%EXPORT_TAGS = ( 'all' => [ qw(
+our @EXPORT_OK = qw(
   convert_to_texinfo
   link_element_to_texi
   target_element_to_texi_label
-) ] );
+);
 
-@EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+our $VERSION = '7.1.90';
 
-$VERSION = '7.1.1';
+my $XS_convert = Texinfo::XSLoader::XS_convert_enabled();
 
+our $module_loaded = 0;
+sub import {
+  if (!$module_loaded) {
+    if ($XS_convert) {
+      # We do not simply override, we must check at runtime
+      # that the document tree was stored by the XS parser.
+      Texinfo::XSLoader::override(
+        "Texinfo::Convert::Texinfo::_convert_tree_with_XS",
+        "Texinfo::Convert::ConvertXS::plain_texinfo_convert_tree"
+      );
+    }
+    $module_loaded = 1;
+  }
+  # The usual import method
+  goto &Exporter::import;
+}
 
 my %brace_commands           = %Texinfo::Commands::brace_commands;
 my %block_commands           = %Texinfo::Commands::block_commands;
@@ -55,11 +77,25 @@ my %def_commands             = %Texinfo::Commands::def_commands;
 # used in root_heading_command_to_texinfo
 my %sectioning_heading_commands = %Texinfo::Commands::sectioning_heading_commands;
 
-my @ignored_types = ('spaces_inserted', 'bracketed_inserted',
-'command_as_argument_inserted');
-my %ignored_types;
-for my $a (@ignored_types) {
-  $ignored_types{$a} = 1;
+# This is used if the document is available for XS, but XS is not
+# used (most likely $TEXINFO_XS_CONVERT is 0).
+sub _convert_tree_with_XS($)
+{
+  my $root = shift;
+
+  return _convert_to_texinfo($root);
+}
+
+sub _convert_to_texinfo($);
+# expand a tree to the corresponding texinfo.
+sub convert_to_texinfo($)
+{
+  my $element = shift;
+
+  if (defined($element->{'tree_document_descriptor'})) {
+    return _convert_tree_with_XS($element);
+  }
+  return _convert_to_texinfo($element);
 }
 
 # TODO document?
@@ -69,12 +105,11 @@ sub link_element_to_texi($)
   my $result = '';
   return $result if (!$element->{'extra'});
   if ($element->{'extra'}->{'manual_content'}) {
-    $result = '('.convert_to_texinfo({'contents'
-                         => $element->{'extra'}->{'manual_content'}}) .')';
+    $result = '('.convert_to_texinfo($element->{'extra'}->{'manual_content'})
+                    .')';
   }
   if ($element->{'extra'}->{'node_content'}) {
-    $result .= convert_to_texinfo({'contents'
-                         => $element->{'extra'}->{'node_content'}});
+    $result .= convert_to_texinfo($element->{'extra'}->{'node_content'});
   }
   return $result
 }
@@ -84,9 +119,49 @@ sub target_element_to_texi_label($)
 {
   my $element = shift;
   my $label_element = Texinfo::Common::get_label_element($element);
+  # get_label_element does not handle links to external manuals in menus
+  if (!defined($label_element)) {
+    return link_element_to_texi($element);
+  }
   return convert_to_texinfo({'contents' => $label_element->{'contents'}});
 }
 
+# only used in Texinfo::Structuring.  Here and not in Texinfo::Structuring
+# to use re => '/a'.
+# $REFERENCE_NODE should always be a target element associated to
+# a label.
+sub check_node_same_texinfo_code($$)
+{
+  my $reference_node = shift;
+  my $node_content = shift;
+
+  my $reference_node_texi;
+  if (defined($reference_node->{'extra'}->{'normalized'})) {
+    my $label_element = Texinfo::Common::get_label_element($reference_node);
+    $reference_node_texi = convert_to_texinfo(
+                                {'contents' => $label_element->{'contents'}});
+    $reference_node_texi =~ s/\s+/ /g;
+  } else {
+    $reference_node_texi = '';
+  }
+
+  my $node_texi;
+  if ($node_content) {
+    my $contents_node = $node_content;
+    if ($node_content->{'contents'}->[-1]->{'type'}
+        and $node_content->{'contents'}->[-1]->{'type'}
+                                      eq 'space_at_end_menu_node') {
+      $contents_node = {'contents' => [@{$node_content->{'contents'}}]};
+      pop @{$contents_node->{'contents'}};
+    }
+    $node_texi = convert_to_texinfo($contents_node);
+    $node_texi =~ s/\s+/ /g;
+  } else {
+    $node_texi = '';
+  }
+  return ($reference_node_texi eq $node_texi);
+}
+
 # for debugging.
 sub root_heading_command_to_texinfo($)
 {
@@ -99,19 +174,20 @@ sub root_heading_command_to_texinfo($)
       $tree = $element->{'args'}->[0]->{'contents'};
     }
   } else {
-    return "Not a root command";
+    return "Not a command";
+  }
+  if ($tree) {
+    return '@'.$element->{'cmdname'}.' '
+                .convert_to_texinfo({'contents' => $tree});
+  } else {
+    return '@'.$element->{'cmdname'};
   }
-  return '@'.$element->{'cmdname'}.' '.convert_to_texinfo({'contents' => $tree})
-          if ($tree);
-  return '@'.$element->{'cmdname'};
 }
 
 # Following subroutines deal with transforming a texinfo tree into texinfo
 # text.  Should give the text that was used parsed, except for a few cases.
 
-# expand a tree to the corresponding texinfo.
-sub convert_to_texinfo($);
-sub convert_to_texinfo($)
+sub _convert_to_texinfo($)
 {
   my $element = shift;
 
@@ -120,8 +196,8 @@ sub convert_to_texinfo($)
      if (ref($element) ne 'HASH');
   my $result = '';
 
-  return '' if ($element->{'type'}
-                and ($ignored_types{$element->{'type'}}));
+  return '' if ($element->{'info'}
+                and $element->{'info'}->{'inserted'});
   if (defined($element->{'text'})) {
     $result .= $element->{'text'};
   } else {
@@ -141,14 +217,14 @@ sub convert_to_texinfo($)
     }
     if (defined($element->{'contents'})) {
       foreach my $child (@{$element->{'contents'}}) {
-        $result .= convert_to_texinfo($child);
+        $result .= _convert_to_texinfo($child);
       }
     }
     if ($element->{'info'} and $element->{'info'}->{'spaces_after_argument'}) {
       $result .= $element->{'info'}->{'spaces_after_argument'}->{'text'};
     }
     if ($element->{'info'} and $element->{'info'}->{'comment_at_end'}) {
-      $result .= convert_to_texinfo($element->{'info'}->{'comment_at_end'})
+      $result .= _convert_to_texinfo($element->{'info'}->{'comment_at_end'})
     }
     $result .= '}' if ($element->{'type'}
                        and ($element->{'type'} eq 'bracketed_arg'
@@ -162,7 +238,7 @@ sub _expand_cmd_args_to_texi($) {
   my $cmd = shift;
 
   my $cmdname = $cmd->{'cmdname'};
-  $cmdname = '' if (!$cmd->{'cmdname'});
+  $cmdname = '' if (! defined($cmd->{'cmdname'}));
   my $result = '';
 
   if ($cmdname) {
@@ -184,9 +260,9 @@ sub _expand_cmd_args_to_texi($) {
     my $braces;
     $braces = 1 if (scalar(@{$cmd->{'args'}})
                     and ($cmd->{'args'}->[0]->{'type'}
-                          and ($cmd->{'args'}->[0]->{'type'} eq 'brace_command_arg'
-                               or $cmd->{'args'}->[0]->{'type'} eq 'brace_command_context'))
-                         or $cmdname eq 'value');
+                          and ($cmd->{'args'}->[0]->{'type'} eq 'brace_container'
+                               or $cmd->{'args'}->[0]->{'type'} eq 'brace_arg'
+                               or $cmd->{'args'}->[0]->{'type'} eq 'brace_command_context')));
     $result .= '{' if ($braces);
     if ($cmdname eq 'verb') {
       $result .= $cmd->{'info'}->{'delimiter'};
@@ -205,12 +281,12 @@ sub _expand_cmd_args_to_texi($) {
     }
     my $arg_nr = 0;
     foreach my $arg (@{$cmd->{'args'}}) {
-      next if $arg->{'type'} and $ignored_types{$arg->{'type'}};
+      next if ($arg->{'info'} and $arg->{'info'}->{'inserted'});
       if ($with_commas) {
         $result .= ',' if ($arg_nr);
         $arg_nr++;
       }
-      $result .= convert_to_texinfo($arg);
+      $result .= _convert_to_texinfo($arg);
     }
     if ($cmdname eq 'verb') {
       $result .= $cmd->{'info'}->{'delimiter'};
